<?php

/*
Plugin Name: WP Site Guardian
Plugin URI: http://pimmpmypay.com
Description: Alerts you of any attempts to compromise your site!
Version: 1.5
Author: Michael Thomas
Author URI: http://pimpmypay.com
*/

define('WPSG_TITLE', 'WP Site Guardian');
define('WPSG_SLUG', 'wpsiteguardian');
define('WPSG_SECRET', 'ad29ac466c11428b');
define('WPSG_PATH', dirname(__FILE__));
define('WPSG_DIR', basename(WPSG_PATH));
define('WPSG_FILE', plugin_basename(__FILE__));
define('WPSG_INCLUDE_DIR', WPSG_PATH . '/include');
define('WPSG_CONTROLLER_DIR', WPSG_PATH . '/controllers');
define('WPSG_VIEW_DIR', WPSG_PATH . '/views');
define('IDS_BASE_DIR', WPSG_INCLUDE_DIR . '/IDS');

require_once WPSG_INCLUDE_DIR . '/Metatick/Metatick_Licensing_1_1.php';
require_once WPSG_INCLUDE_DIR . '/WPSG_Database.php';

class WPSiteGuardian {

	private $licensing = null;
	private $vulnerability_scanner = null;

	/* persistant configuration */
	private $options = null;

	/* defaults */
	private $default_options = array(
		'mode' => 'off',
		'admin_enabled' => false,

		'learning_mode' => false,

		'block_threshold' => 20,
		'notify_threshold' => 20,

		'ban_threshold_total' => 100,
		'ban_threshold_time' => 60,

		'use_reverse_proxy' => false,
		'use_cloudflare' => false,

		'exception_get' => '',
		'exception_post' => '',
		'exception_cookie' => '',
		'exception_server' => '',

		'blocked_message' => 'This request has been blocked',
		'banned_message' => 'You have been banned',

		'vulnerability_scanner_enabled' => true,
		'blacklist_enabled' => true
	);

	/* runtime configuration */
	private $runtime = array();

	/**
	 * WPShieldsUp constructor.
	 */
	function __construct() {
		$this->licensing = new Metatick_Licensing_1_1(WPSG_SLUG, WPSG_SECRET, WPSG_TITLE,true);

		register_activation_hook(__FILE__, array($this, 'activate'));
		register_deactivation_hook(__FILE__, array($this, 'deactivate'));

		if ($this->get_licensing()->is_licensed()) {
			add_action('init', array($this, 'init'), 0);
			if ($this->get_licensing()->has_feature('professional') && $this->get_option('vulnerability_scanner_enabled')) {
				require_once WPSG_INCLUDE_DIR . '/WPSG_VulnerabilityScanner.php';
				$this->vulnerability_scanner = new WPSG_VulnerabilityScanner($this);
			}
		}
		add_action('admin_init', array($this, 'admin_init'));
		add_action('admin_menu', array($this, 'admin_menu'));
	}

	function activate(){
		$this->set_option('mode','off');
	}

	function deactivate(){
		$this->set_option('mode','off');
	}

	function init() {

		if ($this->get_option('mode') != 'off') {
			if (is_admin() && $this->get_option('admin_enabled') == false)
				return;

			$this->run_ids();
		}
	}

	function get_option($name) {
		if ($this->options == null)
			/*if (is_multisite())
				$this->options = get_site_option(WPSG_SLUG, array());
			else*/
				$this->options = get_option(WPSG_SLUG, array());

		if (isset($this->options[$name]))
			return $this->options[$name];

		return $this->default_options[$name];
	}

	function run_ids() {

		require_once WPSG_CONTROLLER_DIR . '/WPSG_BlockedIPController.php';
		require_once WPSG_CONTROLLER_DIR . '/WPSG_WhitelistController.php';
		require_once WPSG_CONTROLLER_DIR . '/WPSG_ThreatLogController.php';

		$whitelist = new WPSG_WhitelistController($this);

		if( current_user_can('manage_options') ){
			$whitelist->add($this->get_current_ip());
		}

		if( $this->get_licensing()->has_feature('professional') && $this->get_option('blacklist_enabled') ){
			require_once WPSG_INCLUDE_DIR . '/WPSG_Blacklist.php';
			$blacklist = new WPSG_Blacklist($this);

			if( $blacklist->is_blocked($this->get_current_ip()) && !$whitelist->is_whitelisted($this->get_current_ip()) ){
				$this->ban_response();
			}
		}

		$controller = new WPSG_BlockedIPController($this);

		require_once IDS_BASE_DIR . '/Init.php';
		require_once IDS_BASE_DIR . '/Log/Composite.php';

		$ids = IDS_Init::init();
		$ids->setConfig($this->get_ids_config(), true);

		$request           = array(
			'GET' => $_GET,
			'POST' => $_POST,
			'COOKIE' => $_COOKIE,
		);
		$request['SERVER'] = array(
			'REQUEST_URI' => (isset($_SERVER['REQUEST_URI'])) ? strtok($_SERVER["REQUEST_URI"], '?') : '',
		);

		$monitor = new IDS_Monitor($request, $ids);

		$result = $monitor->run();

		if ($result->isEmpty()) {
			if ($controller->is_blocked($this->get_current_ip()) && !$whitelist->is_whitelisted($this->get_current_ip())) {
				$this->ban_response();
			}
			return;
		}

		if( current_user_can('manage_options') && $this->get_option('learning_mode') ){
			foreach ($result as $event) {
				$this->add_ids_exception($event->getName());
			}
			return;
		}

		if( $whitelist->is_whitelisted($this->get_current_ip())){
			return;
		}

		$log = new IDS_Log_Composite();

		if ($result->getImpact() > $this->get_option('notify_threshold')) {
			require_once WPSG_INCLUDE_DIR . '/WPSG_IDSEmailer.php';
			$log->addLogger(new WPSG_IDSEmailer($this));
		}

		require_once WPSG_CONTROLLER_DIR . '/WPSG_ThreatLogController.php';
		require_once WPSG_INCLUDE_DIR . '/WPSG_IDSLogger.php';
		$log->addLogger(
			new WPSG_IDSLogger(
				new WPSG_ThreatLogController($this)
			)
		);

		$log->execute($result);


		if ($controller->is_blocked($this->get_current_ip())) {
			$this->ban_response();
		} else {
			$log_controller = new WPSG_ThreatLogController($this);
			$impact         = $log_controller->get_impact_over_time(
				$this->get_current_ip(),
				$this->get_option('ban_threshold_time')
			);

			if ($impact > $this->get_option('ban_threshold_total')) {
				$controller->block($this->get_current_ip());
				$this->ban_response();
			}
		}

		if ($result->getImpact() > $this->get_option('block_threshold')) {
			$this->block_response();
		}

	}

	function get_ids_exceptions(){
		$get = $this->format_exceptions($this->get_option('exception_get'),'GET');
		$post = $this->format_exceptions($this->get_option('exception_post'),'POST');
		$cookie = $this->format_exceptions($this->get_option('exception_cookie'),'COOKIE');
		$server = $this->format_exceptions($this->get_option('exception_server'),'SERVER');

		//prevent people from getting logged out - have to somewhat trust that no plugin is dumb enought to leave a
		//hole open here.. thankfully, very few plugins mess around in this part of WP.

		$hardcoded = array();
		$hardcoded[] = 'POST.log';
		$hardcoded[] = 'POST.pwd';

		$hardcoded[] = 'POST.user_pass';
		$hardcoded[] = 'POST.pass1';
		$hardcoded[] = 'POST.pass2';
		$hardcoded[] = 'POST.password';

		$hardcoded[] = 'COOKIE.__utmz';
		$hardcoded[] = 'COOKIE.__utmc';

		$hardcoded[] = 'COOKIE.s_pers';

		$hardcoded[] = 'POST.permalink_structure';

		return array_merge($get,$post,$cookie,$server,$hardcoded);
	}

	function format_exceptions($exceptions,$parameter){
		$exceptions = str_replace( array( "\r\n", "\r" ), "\n", $exceptions);
		$exceptions = explode( "\n", $exceptions );
		$return = array();
		foreach ( $exceptions as $exception ) {
			$exception = trim($exception);
			if ( strlen($exception) != 0 ) {
				$return[] = $parameter . '.' . $exception;
			}
		}
		return $return;
	}

	function add_ids_exception($exception){
		$dotpos = strpos($exception,'.');
		$array = substr($exception,0,$dotpos);
		$parameter = substr($exception,$dotpos+1);

		switch($array){
			case 'GET':
				$option = 'exception_get';
				break;
			case 'POST':
				$option = 'exception_post';
				break;
			case 'COOKIE':
				$option = 'exception_cookie';
				break;
			case 'SERVER':
				$option = 'exception_server';
				break;
			default:
				return;
		}

		$exceptions = $this->get_option($option);

		$exceptions = str_replace( array( "\r\n", "\r" ), "\n", $exceptions);
		$exceptions = explode( "\n", $exceptions );
		$exceptions[] = $parameter;
		$result = array();
		foreach ( $exceptions as $exception ) {
			$exception = trim($exception);
			if ( strlen($exception) != 0 ) {
				if( !in_array($exception, $result) ){
					$result[] = $exception;
				}
			}
		}
		$exceptions = implode("\n",$result);
		$this->set_option($option,$exceptions);
	}

	function get_ids_config() {
		$dir = wp_upload_dir();
		$tmp_path = $dir['basedir'];

		$config['General']['filter_type']         = 'xml';
		$config['General']['base_path']           = IDS_BASE_DIR;
		$config['General']['use_base_path']       = false;
		$config['General']['filter_path']         = IDS_BASE_DIR . '/default_filter.xml';
		$config['General']['tmp_path']            = $tmp_path;
		$config['General']['scan_keys']           = false;
		$config['General']['HTML_Purifier_Path']  = 'vendors/htmlpurifier/HTMLPurifier.auto.php';
		$config['General']['HTML_Purifier_Cache'] = $tmp_path;
		$config['Caching']['caching']             = 'none';
		$config['General']['exceptions']          = $this->get_ids_exceptions();
		$config['General']['json']                = false;

		return $config;
	}

	function get_current_ip() {
		if ($this->get_option('mode') == 'reverse' && isset($_SERVER["X-Forwarded-For"]))
			return $_SERVER["X-Forwarded-For"];

		if ($this->get_option('mode') == 'cloudflare' && isset($_SERVER["HTTP_CF_CONNECTING_IP"]))
			return $_SERVER["HTTP_CF_CONNECTING_IP"];

		return $_SERVER['REMOTE_ADDR'];
	}

	function ban_response() {
		header('HTTP/1.0 403 Forbidden');
		echo $this->get_option('banned_message');
		exit;
	}

	function block_response() {
		header('HTTP/1.0 403 Forbidden');
		echo $this->get_option('blocked_message');
		exit;
	}

	function admin_init() {
		require_once WPSG_INCLUDE_DIR . '/Metatick/Metatick_Autoupdater_1_2.php';
		new Metatick_Autoupdater_1_2(__FILE__, WPSG_SLUG);

		require_once WPSG_CONTROLLER_DIR . '/WPSG_WhitelistController.php';
		$whitelist = new WPSG_WhitelistController($this);

		if( current_user_can('manage_options') ){
			$whitelist->add($this->get_current_ip());
		}
	}

	function admin_menu() {
		add_menu_page(WPSG_TITLE, WPSG_TITLE, 'manage_options', WPSG_SLUG, array($this, 'render_settings'), plugin_dir_url(__FILE__) . 'assets/icon.png');
		add_submenu_page(WPSG_SLUG, 'Settings', 'Settings', 'manage_options', WPSG_SLUG, array($this, 'render_settings'));
		if ($this->get_licensing()->is_licensed()) {
			if ($this->get_licensing()->has_feature('professional')) {
				add_submenu_page(WPSG_SLUG, WPSG_TITLE . ' Plugins', 'Plugin Vulnerabilities', 'manage_options', WPSG_SLUG . '-plugins', array($this, 'render_plugin_status'));
			}

			add_submenu_page(WPSG_SLUG, WPSG_TITLE . ' Whitelist IPs', 'Whitelisted IPs', 'manage_options', WPSG_SLUG . '-whitelist', array($this, 'render_whitelist'));
			add_submenu_page(WPSG_SLUG, WPSG_TITLE . ' Blocked IPs', 'Blocked IPs', 'manage_options', WPSG_SLUG . '-blocked', array($this, 'render_blocked_ips'));
			add_submenu_page(WPSG_SLUG, WPSG_TITLE . ' Logs', 'Hack Attempt Log', 'manage_options', WPSG_SLUG . '-logs', array($this, 'render_threat_logs'));
		}
		add_submenu_page(WPSG_SLUG, WPSG_TITLE . ' Tutorial', 'Tutorial', 'manage_options', WPSG_SLUG . '-tutorial', array($this, 'render_tutorial'));
	}

	function get_licensing() {
		return $this->licensing;
	}

	function get_vulnerability_scanner() {
		return $this->vulnerability_scanner;
	}

	function set_option($name, $value) {
		if ($this->options == null)
			$this->get_option('mode');

		$this->options[$name] = $value;

		/*if (is_multisite())
			update_site_option(WPSG_SLUG, $this->options);
		else*/
			update_option(WPSG_SLUG, $this->options, true);
	}

	/* Configuration functions for hiding  */

	function get_runtime($name) {
		if (isset($this->runtime[$name]))
			return $this->runtime[$name];

		return $this->default_options[$name];
	}

	function set_runtime($name, $value) {
		$this->runtime[$name] = $value;
	}



	/* misc */



	function configure_runtime() {

	}

	function get_current_username() {
		global $current_user;
		get_currentuserinfo();

		if (!$current_user)
			return '';

		return $current_user->user_login;
	}

	function format_tags($tags, $seperator = '<br />') {
		if (!is_array($tags)) {
			$tags = explode(",", $tags);
		}
		$formatted = '';
		foreach ($tags as $tag) {
			$human_readable = $this->ids_tag_description($tag);
			if ($human_readable == null)
				continue;

			if (strlen($formatted) > 0)
				$formatted .= $seperator;

			$formatted .= $human_readable;
		}
		return $formatted;
	}

	function ids_tag_description($tag) {
		static $tags = array(
			'xss' => 'Cross Site Scripting',
			'csrf' => 'Cross Site Request Forgery',
			'sqli' => 'SQL Injection',
			'dt' => 'Directory Traversal',
			//'id' => '', //Dunno what this is.. hpmh
			'lfi' => 'Local File Inclusion',
			'rfe' => 'Remote File Execution',
			'spam' => 'Spam',
			'dos' => 'Denial Of Service'
		);
		if (isset($tags[$tag]))
			return $tags[$tag];

		return null;
	}

	function render_settings() {
		require_once WPSG_CONTROLLER_DIR . '/WPSG_SettingsController.php';
		$controller = new WPSG_SettingsController($this);
		$controller->handle_actions();
		$controller->render();
	}

	function render_threat_logs() {
		require_once WPSG_CONTROLLER_DIR . '/WPSG_ThreatLogController.php';
		$controller = new WPSG_ThreatLogController($this);
		$controller->handle_actions();
		$controller->render();
	}

	function render_blocked_ips() {
		require_once WPSG_CONTROLLER_DIR . '/WPSG_BlockedIPController.php';
		$controller = new WPSG_BlockedIPController($this);
		$controller->handle_actions();
		$controller->render();
	}

	function render_whitelist() {
		require_once WPSG_CONTROLLER_DIR . '/WPSG_WhitelistController.php';
		$controller = new WPSG_WhitelistController($this);
		$controller->handle_actions();
		$controller->render();
	}

	function render_plugin_status() {
		require_once WPSG_CONTROLLER_DIR . '/WPSG_PluginVulnerabilitiesController.php';
		$controller = new WPSG_PluginVulnerabilitiesController($this);
		$controller->handle_actions();
		$controller->render();
	}

	function render_tutorial() {
		require_once WPSG_VIEW_DIR . '/WPSG_TutorialView.php';
		$view = new WPSG_TutorialView($this);
		$view->render();
	}
}

$WPSiteGuardian = new WPSiteGuardian();