<?php
/**
 * Created by PhpStorm.
 * User: michael
 * Date: 24/11/2015
 * Time: 1:20 PM
 */

require_once WPSG_INCLUDE_DIR . '/WPSG_Database.php';

class WPSG_BlockedIPController {

	protected $wpsg = null;

	public function __construct($wpsg){
		$this->wpsg = $wpsg;
	}

	public function is_blocked($ip){
		global $wpdb;
		$table_name = WPSG_Database::get_blocked_ips_table_name();

		$count = $wpdb->get_var(
			$wpdb->prepare(	"
				SELECT count(ip)
				FROM $table_name
				WHERE ip = %s
				",
				$ip
			)
		);

		return ! ($count == null || $count == 0);
	}

	public function block($ip) {

		$ip =  filter_var($ip, FILTER_VALIDATE_IP);

		if( $ip == false )
			return;

		global $wpdb;

		if( $this->is_blocked($ip) )
			return;

		$wpdb->insert(
				WPSG_Database::get_blocked_ips_table_name(),
			array(
				'date' => date( 'Y-m-d H:i:s', time() ),
				'ip' => $ip,
			)
		);
	}

	public function delete($ids){
		if( ! is_array($ids) ){
			$ids = array($ids);
		}
		global $wpdb;
		foreach($ids as $id) {
			$wpdb->delete(
					WPSG_Database::get_blocked_ips_table_name(),
					array(
							'id' => $id
					),
					array('%s')
			);
		}
	}

	public function delete_all(){
		global $wpdb;
		$sql = 'DELETE FROM ' . WPSG_Database::get_blocked_ips_table_name();
		$wpdb->query( $sql );
	}

	public function handle_actions() {
		if( current_user_can('manage_options') ) {
			if (isset($_POST['action']) && $_POST['action'] == 'add') {
				if (wp_verify_nonce($_POST['add-blocklist'], 'add')) {
					$ip = $_POST['add_blocklist_ip'];
					$this->block($ip);
				}
			}

			if (isset($_POST['action']) && $_POST['action'] == 'delete') {
				if (wp_verify_nonce($_POST['_wpnonce'], 'bulk-blocks')) {
					$this->delete($_POST['id']);
				}
			}
		}
	}

	public function render(){
		global $wpdb;
		$query = 'SELECT * FROM ' . WPSG_Database::get_blocked_ips_table_name();

		$search = !empty($_REQUEST['s']) ? $_REQUEST['s'] : '';

		if(!empty($search)){
			$query .= $wpdb->prepare( ' WHERE ip LIKE %s', '%' . $wpdb->esc_like($search) . '%');
		}

		$orderbys = array(
			'date',
			'ip'
		);

		$orders = array(
			'asc',
			'desc'
		);

		$orderby = 'date';
		$order = '';

		if( !empty($_GET['orderby']) ){
			if( in_array($_GET['orderby'],$orderbys)){
				$orderby = $_GET['orderby'];
			}
		}

		if( !empty($_GET['order']) ){
			if( in_array($_GET['order'],$orders)){
				$order = $_GET['order'];
			}
		}

		if(!empty($orderby) && !empty($order)){
			$query.=' ORDER BY '.$orderby.' '.$order;
		}

		$total = $wpdb->query($query);
		$per_page = 25;

		$paged = !empty($_GET['paged']) ? esc_sql($_GET['paged']) : '';

		if(empty($paged) || !is_numeric($paged) || $paged<=0 ){
			$paged=1;
		}

		if(!empty($paged) && !empty($per_page)){
			$offset=($paged-1)*$per_page;
			$query.=' LIMIT '.(int)$offset.','.(int)$per_page;
		}

		$items = $wpdb->get_results($query);

		require_once WPSG_VIEW_DIR . '/WPSG_BlockedIPView.php';
		$view = new WPSG_BlockedIPView($this->wpsg);
		return $view->render($items,$total,$per_page);
	}
}
