<?php
/**
 * Created by PhpStorm.
 * User: michael
 * Date: 24/11/2015
 * Time: 1:20 PM
 */

require_once WPSG_INCLUDE_DIR . '/WPSG_Database.php';

class WPSG_ThreatLogController {

	protected $wpsg = null;

	public function __construct($wpsg) {
		$this->wpsg = $wpsg;
	}

	public function log_ids_report(IDS_Report $report) {

		global $wpdb;

		$url = $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];

		if( $_SERVER['REQUEST_SCHEME'] == null || empty($_SERVER['REQUEST_SCHEME']) )
			$url = 'http' . $url;

		$referer = '';
		if (isset($_SERVER['HTTP_REFERER']))
			$referer = sanitize_text_field($_SERVER['HTTP_REFERER']);

		foreach ($report as $event) {
			$wpdb->insert(
					WPSG_Database::get_log_table_name(),
					array(
							'date' => date('Y-m-d H:i:s', time()),
							'name' => $event->getName(),
							'value' => $event->getValue(),
							'tags' => implode(',', $event->getTags()),
							'impact' => $event->getImpact(),
							'total_impact' => $report->getImpact(),
							'url' => $url,
							'referrer' => $referer,

							'ip' => $this->wpsg->get_current_ip(),
							'username' => $this->wpsg->get_current_username()
					)
			);
		}
		return true;
	}

	public function delete_logs($ids){
		global $wpdb;
		foreach($ids as $id){
			$wpdb->delete(
					WPSG_Database::get_log_table_name(),
					array(
							'ID' => $id
					),
					array( '%d' )
			);
		}
	}

	public function add_exceptions($ids){
		global $wpdb;
		$table = WPSG_Database::get_log_table_name();
		foreach($ids as $id){
			$parameter = $wpdb->get_var(
					$wpdb->prepare("SELECT name FROM $table WHERE id = %d", $id)
			);

			$this->wpsg->add_ids_exception($parameter);
		}
	}

	public function handle_actions() {
		if( current_user_can('manage_options') ) {
			if (isset($_POST['action']) && $_POST['action'] == 'delete') {
				if (wp_verify_nonce($_POST['_wpnonce'], 'bulk-logs')) {
					$this->delete_logs($_POST['id']);
				}
			}
			if (isset($_POST['action']) && $_POST['action'] == 'except') {
				if (wp_verify_nonce($_POST['_wpnonce'], 'bulk-logs')) {
					$this->add_exceptions($_POST['id']);
				}
			}
		}
	}

	public function get_impact_over_time($ip, $minutes){
		global $wpdb;

		$query = 'SELECT sum(impact) impact FROM ' . WPSG_Database::get_log_table_name();
		$query .= $wpdb->prepare( ' WHERE ip = %s', $ip );

		$query .= $wpdb->prepare( ' AND date > DATE_SUB(NOW(), INTERVAL %d MINUTE)', $minutes );

		return $wpdb->get_var($query);
	}

	public function render(){
		global $wpdb;
		$query = 'SELECT * FROM ' . WPSG_Database::get_log_table_name();

		$search = !empty($_REQUEST['s']) ? $_REQUEST['s'] : '';

		if(!empty($search)){
			$query .= $wpdb->prepare( ' WHERE ip LIKE %s', '%' . $wpdb->esc_like($search) . '%');
			$query .= $wpdb->prepare( ' OR url LIKE %s', '%' . $wpdb->esc_like($search) . '%');
			$query .= $wpdb->prepare( ' OR value LIKE %s', '%' . $wpdb->esc_like($search) . '%');
		}

		$orderbys = array(
				'date',
				'ip',
				'name',
				'url'
		);

		$orders = array(
				'asc',
				'desc'
		);

		$orderby = 'date';
		$order = '';

		if( !empty($_GET['orderby']) ){
			if( in_array($_GET['orderby'],$orderbys)){
				$orderby = $_GET['orderby'];
			}
		}

		if( !empty($_GET['order']) ){
			if( in_array($_GET['order'],$orders)){
				$order = $_GET['order'];
			}
		}

		if(!empty($orderby) && !empty($order)){
			$query.=' ORDER BY '.$orderby.' '.$order;
		}

		$total = $wpdb->query($query);
		$per_page = 25;

		$paged = !empty($_GET['paged']) ? esc_sql($_GET['paged']) : '';

		if(empty($paged) || !is_numeric($paged) || $paged<=0 ){
			$paged=1;
		}

		if(!empty($paged) && !empty($per_page)){
			$offset=($paged-1)*$per_page;
			$query.=' LIMIT '.(int)$offset.','.(int)$per_page;
		}

		$items = $wpdb->get_results($query);

		require_once WPSG_VIEW_DIR . '/WPSG_ThreatLogView.php';
		$view = new WPSG_ThreatLogView($this->wpsg);
		return $view->render($items,$total,$per_page);
	}
}
