<?php
/**
 * Created by PhpStorm.
 * User: michael
 * Date: 27/11/2015
 * Time: 10:53 AM
 */

if( ! class_exists('Metatick_Licensing_1_0') ){
	class Metatick_Licensing_1_0{
		protected $product = null;
		protected $secret = null;
		protected $name = null;
		protected $options = null;
		protected $per_site_licensing = false;

		protected $changed = false;

		private $default_options = array(
			'error' => null,
			'license' => '',
			'expire' => '0000-00-00',
			'verify' => '',
			'notified' => false,
			'features' => array()
		);


		public function __construct($product,$secret,$name,$per_site_licensing = false){
			$this->product = $product;
			$this->secret = $secret;
			$this->name = $name;
			$this->per_site_licensing = $per_site_licensing;
			add_action( 'admin_notices', array($this,'admin_notices') );
			if ( ! wp_next_scheduled( 'Metatick_Licensing_do_check' ) ) {
				wp_schedule_event( time(), 'hourly', 'Metatick_Licensing_do_check' );
			}
			add_action( 'Metatick_Licensing_do_check', array($this,'maybe_update'));
		}

		public function admin_notices(){
			if( $this->is_licensed() ){
				return;
			}
			$message = "{$this->name} is unlicensed";

			if( $this->has_error() )
				$message .= ': ' . $this->get_error();

			echo '<div class="error">';
			echo "<p>$message</p>";
			echo '</div>';
		}

		function set_option($name, $value) {
			if ($this->options == null)
				$this->get_option('license');

			$this->options[$name] = $value;

			if (is_multisite() && !$this->per_site_licensing)
				update_site_option($this->product . '_licensing', $this->options);
			else
				update_option($this->product . '_licensing', $this->options, true);
		}

		function get_option($name) {
			if ($this->options == null)
				if (is_multisite() && !$this->per_site_licensing)
					$this->options = get_site_option($this->product . '_licensing', array());
				else
					$this->options = get_option($this->product . '_licensing', array());

			if (isset($this->options[$name]))
				return $this->options[$name];

			return $this->default_options[$name];
		}

		function get_license(){
			return $this->get_option('license');
		}

		function set_license($license){
			if( $license != $this->get_license() ){
				$this->changed = true;
			}
			$this->set_option('license',$license);

			if( $this->changed )
				$this->update();
		}

		function get_expire(){
			return $this->get_option('expire');
		}

		function set_expire($expire){
			$this->set_option('expire',$expire);
		}

		function get_verify(){
			return $this->get_option('verify');
		}

		function set_verify($verify){
			$this->set_option('verify',$verify);
		}

		function get_error(){
			return $this->get_option('error');
		}

		function has_error(){
			$error = $this->get_option('error');

			if( $error != null )
				return ! empty($error);

			return false;
		}

		function set_error($error){
			$this->set_option('error',$error);
		}

		function get_features(){
			return $this->get_option('features');
		}

		function set_features($features){
			$this->set_option('features',$features);
		}

		function get_uid(){
			$data = AUTH_KEY . get_current_blog_id();
			$hash = strtolower(md5(strtolower($data)));
			return $hash;
		}

		function is_licensed(){
			//license expired.. doh!
			if( $this->get_remaining_seconds() < 0 ){
				return false;
			}

			$data = $this->get_uid() . $this->get_license() . $this->secret . $this->get_expire();
			$hash = strtolower(md5(strtolower($data)));
			return $hash === $this->get_verify();
		}

		function update_features(){
			try{
				$url = "http://licensing.metatick.net/features/{$this->product}/{$this->get_license()}";
				$response = wp_remote_get($url);
				$json = json_decode(wp_remote_retrieve_body( $response ));
				if( isset($json->result) ){
					if( $json->result === 'success' ){
						$this->set_features($json->features);
					}
				}
			}catch (Exception $e){
			}
		}

		function update(){
			try{
				$url = "http://licensing.metatick.net/check/{$this->product}/{$this->get_license()}/{$this->get_uid()}";
				$response = wp_remote_get($url);
				$json = json_decode(wp_remote_retrieve_body( $response ));
				if( isset($json->result) ){
					if( $json->result === 'success' ){
						$this->set_expire($json->expire);
						$this->set_verify($json->verify);
						$this->set_error(null);

						$this->update_features();
					}
					if( $json->result === 'error' )
					{
						if( isset($json->remove) )
						{
							$this->set_expire('2001-01-01');
							$this->set_verify('');
						}
						throw new Exception($json->error);
					}
				}
				throw new Exception("Unknown Error");
			}catch (Exception $e){
				$this->set_error($e->getMessage());
			}
		}

		function maybe_update(){
			$remaining = $this->get_remaining_seconds();
			if( $remaining < (7 * 24 * 60 * 60) ) {
				try	{
					$result = $this->update();
					$this->set_option('notified',false);
				}catch(Exception $e){
					if( $remaining <= 0 && $this->get_option('notified') == false )	{
						notify( '[' . $this->productname . '] Licensing Error', "Product has expired");
						$this->set_option('notified',true);
					} else if( $remaining > 0 )	{
						notify( '[' . $this->productname . '] Licensing Error', "Product will cease to function in " . ($remaining / (60 * 60)) . " hours");
					}
				}
			}
		}

		function notify($subject,$email){
			wp_mail( get_option('admin_email'), $subject, $email );
		}

		function has_changed(){
			return $this->changed;
		}

		function get_remaining_seconds(){
			$expire = new DateTime($this->get_expire());
			$now = new DateTime();
			return $expire->format('U') - $now->format('U');
		}

		function get_status(){
			if( $this->is_licensed() ){
				return 'Licensed';
			}
			if( $this->has_error() )
				return $this->get_error();

			return 'Not Licensed';
		}

		function has_feature($check){
			foreach( $this->get_features() as $feature ){
				if( $check === $feature )
					return true;
			}
			return false;
		}
	}
}