<?php
/**
 * Created by PhpStorm.
 * User: michael
 * Date: 25/11/2015
 * Time: 7:00 PM
 */

define('WPSG_DB_VERSION', '1.1');

class WPSG_Database {

	public static function log_version_1_0() {

		$table_name = WPSG_Database::get_log_table_name();

		$sql = "CREATE TABLE $table_name (
					id mediumint(9) NOT NULL AUTO_INCREMENT,
					date datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
					name varchar(255) NOT NULL,
					value text NOT NULL,
					tags varchar(255) NOT NULL,
					impact int(10) NOT NULL,
					total_impact int(10) NOT NULL,
					url text NOT NULL,
					referrer text NULL,
					ip varchar(32) NOT NULL,
					username varchar(255) NOT NULL,
					UNIQUE KEY id (id)
				);";

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);
	}

	public static function blocked_ips_version_1_0(){

		$table_name = WPSG_Database::get_blocked_ips_table_name();

		$sql = "CREATE TABLE $table_name (
					id mediumint(9) NOT NULL AUTO_INCREMENT,
					date datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
					ip varchar(32) NOT NULL,
					UNIQUE KEY ip (ip),
					UNIQUE KEY id (id)
				);";

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta( $sql );
	}

	public static function blacklist_version_1_0(){

		$table_name = WPSG_Database::get_blacklist_table_name();

		$sql = "CREATE TABLE $table_name (
					id mediumint(9) NOT NULL AUTO_INCREMENT,
					ip varchar(32) NOT NULL,
					UNIQUE KEY ip (ip),
					UNIQUE KEY id (id)
				);";

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta( $sql );
	}

	public static function whtielist_version_1_1(){

		$table_name = WPSG_Database::get_whitelist_table_name();

		$sql = "CREATE TABLE $table_name (
					id mediumint(9) NOT NULL AUTO_INCREMENT,
					ip varchar(32) NOT NULL,
					UNIQUE KEY ip (ip),
					UNIQUE KEY id (id)
				);";

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta( $sql );
	}

	public static function install_1_0(){
		WPSG_Database::log_version_1_0();
		WPSG_Database::blocked_ips_version_1_0();
		WPSG_Database::blacklist_version_1_0();
		update_site_option("wpsg_db_version", '1.0');
	}

	public static function install_1_1(){
		WPSG_Database::whtielist_version_1_1();
		update_site_option("wpsg_db_version", '1.1');
	}

	public static function check() {
		if (get_site_option('wpsg_db_version') == WPSG_DB_VERSION)
			return;

		if (get_site_option('wpsg_db_version') == null) {
			update_site_option("wpsg_db_version", '0.0');
		}

		if (version_compare(get_site_option('wpsg_db_version'), '1.0', '<')) {
			WPSG_Database::install_1_0();
		}
		if (version_compare(get_site_option('wpsg_db_version'), '1.1', '<')) {
			WPSG_Database::install_1_1();
		}


		/* this is for "downgrades". we just kinda hop dbDelta is smart enough... */
		if (get_site_option('wpsg_db_version') != WPSG_DB_VERSION)
			WPSG_Database::install_1_1();
	}



	public static function get_log_table_name() {
		global $wpdb;
		return $wpdb->prefix . 'wpsg_log';
	}

	public static function get_blocked_ips_table_name() {
		global $wpdb;
		return $wpdb->prefix . 'wpsg_blocked_ips';
	}

	public static function get_blacklist_table_name() {
		global $wpdb;
		return $wpdb->prefix . 'wpsg_blacklist';
	}

	public static function get_whitelist_table_name() {
		global $wpdb;
		return $wpdb->prefix . 'wpsg_whitelist';
	}


}

WPSG_Database::check();