<?php
/**
 * Created by PhpStorm.
 * User: michael
 * Date: 12/11/2015
 * Time: 3:40 PM
 */

require_once WPSG_INCLUDE_DIR . '/Metatick/Metatick_WPSettingsHelper_1_0.php';
require_once WPSG_INCLUDE_DIR . '/Metatick/Metatick_WPSingleSettingsPage_1_0.php';
require_once WPSG_INCLUDE_DIR . '/Metatick/Metatick_WPTabbedSettingsPage_1_0.php';

class WPSG_SettingsView {
	private $wpsg = null;
	private $tabhelper = null;

	public function __construct($wpsg) {
		$this->wpsg       = $wpsg;

		$this->tabhelper   = new Metatick_WPTabbedSettingsPage_1_0(WPSG_TITLE, 'configuration');
		$this->tabhelper->add_tab('configuration', 'Configuration', array($this, 'fill_configuration'));
		$this->tabhelper->add_tab('advanced', 'Advanced', array($this, 'fill_advanced'));
		$this->tabhelper->add_tab('exceptions', 'Exceptions', array($this, 'fill_exceptions'));
	}

	public function fill_configuration() {
		add_settings_section(
				'wpsg_settings',
				'Configuration',
				array('Metatick_WPSettingsHelper_1_0', 'render_section'),
				WPSG_SLUG
		);

		add_settings_field(
				'license_key',
				'License Key',
				array('Metatick_WPSettingsHelper_1_0', 'render_textbox'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'license_key',
						'id' => 'license_key',
						'description' => 'Enter your license here',
						'value' => $this->wpsg->get_licensing()->get_license(),
				)
		);

		add_settings_field(
				'mode',
				'Mode',
				array('Metatick_WPSettingsHelper_1_0', 'render_radio'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'mode',
						'id' => 'mode',
						'value' => $this->wpsg->get_option('mode'),
						'options' => array(
								'off' => 'Off',
								'enabled' => 'Enabled (Most sites)',
								'reverse' => 'Enabled Reverse Proxy Mode (Sites behind a reverse proxy)',
								'cloudflare' => 'Enabled CloudFlare Mode (Sites behind CloudFlare)'
						)
				)
		);

		add_settings_field(
				'blocked_message',
				'Blocked Message',
				array('Metatick_WPSettingsHelper_1_0', 'render_textbox'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'blocked_message',
						'id' => 'blocked_message',
						'description' => 'Message when a visitor action is blocked',
						'value' => $this->wpsg->get_option('blocked_message'),
				)
		);

		add_settings_field(
				'banned_message',
				'Banned Message',
				array('Metatick_WPSettingsHelper_1_0', 'render_textbox'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'banned_message',
						'id' => 'banned_message',
						'description' => 'Message when a visitor action is blocked',
						'value' => $this->wpsg->get_option('banned_message'),
				)
		);

		return 'wpsg_configuration';
	}

	public function fill_advanced() {
		add_settings_section(
				'wpsg_settings',
				'Advanced Configuration',
				array('Metatick_WPSettingsHelper_1_0', 'render_section'),
				WPSG_SLUG
		);

		if( $this->wpsg->get_licensing()->has_feature('professional') ){
			add_settings_field(
					'vulnerability_scanner_enabled',
					'Enable Vulnerability Scanning',
					array('Metatick_WPSettingsHelper_1_0', 'render_checkbox'),
					WPSG_SLUG,
					'wpsg_settings',
					array(
							'name' => 'vulnerability_scanner_enabled',
							'id' => 'vulnerability_scanner_enabled',
							'description' => 'Check for known vulnerable plugins',
							'value' => $this->wpsg->get_option('vulnerability_scanner_enabled'),
					)
			);
			add_settings_field(
					'blacklist_enabled',
					'Live Blacklist',
					array('Metatick_WPSettingsHelper_1_0', 'render_checkbox'),
					WPSG_SLUG,
					'wpsg_settings',
					array(
							'name' => 'blacklist_enabled',
							'id' => 'blacklist_enabled',
							'description' => 'Enable live blacklist for "bad" IP addresses',
							'value' => $this->wpsg->get_option('blacklist_enabled'),
					)
			);
		}

		add_settings_field(
				'admin_enabled',
				'Enable Admin Checking',
				array('Metatick_WPSettingsHelper_1_0', 'render_checkbox'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'admin_enabled',
						'id' => 'admin_enabled',
						'description' => 'Enable for admin site',
						'value' => $this->wpsg->get_option('admin_enabled'),
				)
		);

		add_settings_field(
				'learning_mode',
				'Enable Learning Mode',
				array('Metatick_WPSettingsHelper_1_0', 'render_checkbox'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'learning_mode',
						'id' => 'learning_mode',
						'description' => 'Automatically add exceptions for administrator accounts',
						'value' => $this->wpsg->get_option('learning_mode'),
				)
		);

		add_settings_field(
				'block_threshold',
				'Block Threshold',
				array('Metatick_WPSettingsHelper_1_0', 'render_textbox'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'block_threshold',
						'id' => 'block_threshold',
						'description' => 'Block all requests with an impact higher than this',
						'value' => $this->wpsg->get_option('block_threshold'),
				)
		);

		add_settings_field(
				'notify_threshold',
				'Email Threshold',
				array('Metatick_WPSettingsHelper_1_0', 'render_textbox'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'notify_threshold',
						'id' => 'notify_threshold',
						'description' => 'Send an email notification for all requests with an impact higher than this',
						'value' => $this->wpsg->get_option('notify_threshold'),
				)
		);

		add_settings_field(
				'ban_threshold_total',
				'Ban Threshold',
				array($this, 'render_ban_threshold'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'ban_threshold_total',
						'id' => 'ban_threshold_total',
						'description' => 'Send an email notification for all requests with an impact higher than this',
						'value' => $this->wpsg->get_option('ban_threshold_total'),
				)
		);

		return 'wpsg_advanced';
	}

	public function fill_exceptions() {
		add_settings_section(
				'wpsg_settings',
				'Exceptions',
				array('Metatick_WPSettingsHelper_1_0', 'render_section'),
				WPSG_SLUG
		);

		add_settings_field(
				'exception_get',
				'GET Exceptions',
				array('Metatick_WPSettingsHelper_1_0', 'render_textarea'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'exception_get',
						'id' => 'exception_get',
						'description' => '',
						'value' => $this->wpsg->get_option('exception_get'),
						'width' => '400px',
						'height' => '120px'
				)
		);

		add_settings_field(
				'exception_post',
				'POST Exceptions',
				array('Metatick_WPSettingsHelper_1_0', 'render_textarea'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'exception_post',
						'id' => 'exception_post',
						'description' => '',
						'value' => $this->wpsg->get_option('exception_post'),
						'width' => '400px',
						'height' => '120px'
				)
		);

		add_settings_field(
				'exception_cookie',
				'COOKIE Exceptions',
				array('Metatick_WPSettingsHelper_1_0', 'render_textarea'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'exception_cookie',
						'id' => 'exception_cookie',
						'description' => '',
						'value' => $this->wpsg->get_option('exception_cookie'),
						'width' => '400px',
						'height' => '120px'
				)
		);

		add_settings_field(
				'exception_server',
				'SERVER Exceptions',
				array('Metatick_WPSettingsHelper_1_0', 'render_textarea'),
				WPSG_SLUG,
				'wpsg_settings',
				array(
						'name' => 'exception_server',
						'id' => 'exception_server',
						'description' => '',
						'value' => $this->wpsg->get_option('exception_server'),
						'width' => '400px',
						'height' => '120px'
				)
		);

		return 'wpsg_exceptions';
	}

	function render_ban_threshold($args) {
		echo '<p>';
		echo 'Ban IP after ';
		echo "<input type='text' class='text' id='ban_threshold_total' name='ban_threshold_total' value='{$this->wpsg->get_option('ban_threshold_total')}'/>";
		echo 'total impact within';
		echo "<input type='text' class='text' id='ban_threshold_time' name='ban_threshold_time' value='{$this->wpsg->get_option('ban_threshold_time')}'/>";
		echo 'minutes';
		echo '</p>';
	}

	public function render() {
		$this->tabhelper->render(WPSG_SLUG);
		?>
			<script>
				var mode_warning = false;
				jQuery(function(){
					current_mode = jQuery('input[name="mode"]:checked').val();

					jQuery('input[name="mode"]').change(function(){
						if( !mode_warning )
							jQuery('<p style="color: red;">Changing your mode will reset yout blocked IPs, Whitelist &amp; add your current IP to the whitelist</p>').appendTo(jQuery('input[name="mode"]').last().parent());

						mode_warning = true;
					});
				});
			</script>
		<?php
	}
}