<?php
/**
 * Created by PhpStorm.
 * User: michael
 * Date: 24/11/2015
 * Time: 5:22 PM
 */

class WPSG_PluginVulnerabilitiesTable extends WP_List_Table {

	protected $wpsg = null;
	function __construct($wpsg,$items) {
		parent::__construct( array(
			'singular'=> 'log', //Singular label
			'plural' => 'logs', //plural label, also this well be one of the table css class
			'ajax'   => false //We won't support Ajax for this table
		) );

		$this->wpsg = $wpsg;
		$this->items = $items;
		$this->set_pagination_args( array(
				"total_items" => count($items),
				"total_pages" => 1,
				"per_page" => count($items),
		) );
		$this->_column_headers=array($this->get_columns(), array(), $this->get_sortable_columns());
	}

	function get_columns() {
		return $columns= array(
			'plugin'=>__('Plugin'),
			'version'=>__('Version'),
			'vulnerabilities'=>__('Vulnerabilities'),
		);
	}

	public function get_sortable_columns() {
		return array();
	}

	function column_plugin($item) {
		return $item['data']['Name'];
	}

	function column_version($item) {
		return $item['data']['Version'];
	}

	function column_vulnerabilities($item) {
		$html = '';
		if( $item['vulnerabilities'] != null ) {
			foreach ($item['vulnerabilities'] as $vulnerability) {
				$fixed_in   = $vulnerability['fixed_in'];
				$fixed = false;
				$applies = true;
				if ($fixed_in != null) {
					if (version_compare($item['data']['Version'], $fixed_in, '>=')) {
						$fixed = true;
					}
				}
				if (!$fixed) {
					$url   = null;
					$title = $vulnerability['title'];
					if (isset($vulnerability['references']['url']) && isset($vulnerability['references']['url'][0])) {
						$url = $vulnerability['references']['url'][0];
					}
					if ($url == null) {
						$html .= "$title<br />";
					} else {
						$html .= "<a href='$url'>$title</a><br />";
					}
				}
			}
		}
		if( strlen($html) == 0 )
			return 'No known vulnerabilities';

		return $html;
	}

	public function is_vulnerable($item){
		if( $item['vulnerabilities'] != null ) {
			foreach ($item['vulnerabilities'] as $vulnerability) {
				$fixed_in   = $vulnerability['fixed_in'];
				$fixed = false;
				$applies = true;
				if ($fixed_in != null) {
					if (version_compare($item['data']['Version'], $fixed_in, '>=')) {
						$fixed = true;
					}
				}
				if (!$fixed) {
					return true;
				}
			}
		}
		return false;
	}

	public function single_row( $item ) {
		if( $this->is_vulnerable($item) ){
			echo '<tr style="background-color:#FFD6D6;">';
		}else{
			echo '<tr style="background-color:#D6FFE4;">';
		}
		$this->single_row_columns( $item );
		echo '</tr>';
	}
}