<?php
/**
 * Created by PhpStorm.
 * User: michael
 * Date: 28/11/2015
 * Time: 9:15 AM
 */

class WPSG_VulnerabilityScanner{
	protected $wpsg;

	public function __construct($wpsg){
		$this->wpsg = $wpsg;

		if ($this->wpsg->get_licensing()->is_licensed()) {
			if ($this->wpsg->get_licensing()->has_feature('professional')) {
				if (!wp_next_scheduled(WPSG_SLUG . '_vulnerable_plugin_check')) {
					wp_schedule_event(time(), 'hourly', WPSG_SLUG . '_vulnerable_plugin_check');
				}
				add_action(WPSG_SLUG . '_vulnerable_plugin_check', array($this, 'scan_and_notify'));
			}
		}
	}

	public function scan_and_notify(){
		$notified = array();
		$plugins  = $this->get_plugins();

		foreach ($plugins as $plugin) {
			if ($plugin['vulnerabilities'] != null) {
				foreach ($plugin['vulnerabilities'] as $vulnerability) {
					$fixed_in = $vulnerability['fixed_in'];
					$fixed    = false;
					$applies  = true;
					if ($fixed_in != null) {
						if (version_compare($plugin['data']['Version'], $fixed_in, '>=')) {
							$fixed = true;
						}
					}
					if (!$fixed) {
						$this->maybe_notify($plugin, $vulnerability);
						$notified[] = $vulnerability['id'];
					}
				}
			}
		}
		$this->set_notified($notified);
	}

	function maybe_notify($plugin, $vulnerability) {
		if ($this->have_notified($vulnerability['id']))
			return;

		$name    = get_bloginfo('name');
		$subject = "[$name] ALERT: Vulnerable Plugin Detected: {$plugin['data']['Name']}";
		$url     = $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];

		$url = null;
		if (isset($vulnerability['references']['url']) && isset($vulnerability['references']['url'][0])) {
			$url = $vulnerability['references']['url'][0];
		}

		$title = $vulnerability['title'];

		$email = 'Plugin: ' . $plugin['data']['Name'] . "\n";
		$email .= 'Version: ' . $plugin['data']['Version'] . "\n";
		$email .= 'Vulnerability : ' . $title . "\n";
		if ($url != null)
			$email .= 'Read More: ' . $url . "\n";

		$email .= "\n";
		$email .= "\n";
		$email .= "\n";
		$email .= 'It is recommended that you update this plugin or disable it if an update is not available.';

		wp_mail(get_option('admin_email'), $subject, $email);
	}

	public function get_plugins() {
		if (!function_exists('get_plugins'))
			include ABSPATH . 'wp-admin/includes/plugin.php';

		$items   = array();
		$plugins = get_plugins();

		foreach ($plugins as $file => $data) {
			if (strpos($file, '/') === false) {
				$pos = strrpos($file, '.');
				if ($pos === false) {
					$slug = $file;
				} else {
					$slug = substr($file, 0, $pos);
				}
			} else {
				$slug = strtok($file, '/');
			}
			$items[$slug]                    = array();
			$items[$slug]['vulnerabilities'] = null;

			$vulnerabilities = $this->get_plugin_vulnerabilities($slug);
			if (isset($vulnerabilities[$slug])) {
				if (isset($vulnerabilities[$slug]['vulnerabilities'])) {
					$items[$slug]['vulnerabilities'] = $vulnerabilities[$slug]['vulnerabilities'];
				}
			}
			$items[$slug]['data'] = $data;
			$items[$slug]['slug'] = $slug;
		}
		return $items;
	}
	function get_plugin_vulnerability_cache() {
		return get_option(WPSG_SLUG . '_plugin_vulnerabilities', array());
	}

	function set_plugin_vulnerability_cache($vulnerabilities) {
		update_option(WPSG_SLUG . '_plugin_vulnerabilities', $vulnerabilities);
	}

	function get_notified() {
		return get_option(WPSG_SLUG . '_plugin_vulnerability_notifications', array());
	}

	function set_notified($notifications) {
		update_option(WPSG_SLUG . '_plugin_vulnerability_notifications', $notifications);
	}

	function have_notified($id) {
		foreach ($this->get_notified() as $notified_id) {
			if ($id === $notified_id)
				return true;
		}
		return false;
	}

	public function get_plugin_vulnerabilities($plugin_slug) {

		$plugin_vulnerabilities = $this->get_plugin_vulnerability_cache();

		$last_update = 0;

		if (isset($plugin_vulnerabilities[$plugin_slug])) {
			$last_update = $plugin_vulnerabilities[$plugin_slug]['last_update'];
		}

		if (($last_update + (24 * 60 * 60)) < time()) {
			$plugin_vulnerabilities[$plugin_slug]                = array();
			$plugin_vulnerabilities[$plugin_slug]['last_update'] = time();
			$plugin_vulnerabilities[$plugin_slug]['data']        = null;

			$url      = "https://wpvulndb.com/api/v2/plugins/$plugin_slug";
			$response = wp_remote_get($url);
			if (is_array($response)) {
				if ($response['response']['code'] == 200) {
					$data                                         = json_decode($response['body'], true);
					$plugin_vulnerabilities[$plugin_slug]['data'] = $data;
				}
			}
			$this->set_plugin_vulnerability_cache($plugin_vulnerabilities);
		}
		return $plugin_vulnerabilities[$plugin_slug]['data'];
	}
}